@@ -64,7 +64,7 @@ intel_update_max_level(struct intel_texture_object *intelObj,
  * BaseLevel/MaxLevel/filtering, and copy in any texture images that are
  * stored in other miptrees.
  */
-GLuint
+void
 intel_finalize_mipmap_tree(struct brw_context *brw, GLuint unit)
 {
    struct gl_context *ctx = &brw->ctx;
@@ -78,7 +78,7 @@ intel_finalize_mipmap_tree(struct brw_context *brw, GLuint unit)
 
    /* TBOs require no validation -- they always just point to their BO. */
    if (tObj->Target == GL_TEXTURE_BUFFER)
-      return true;
+      return;
 
    /* We know that this is true by now, and if it wasn't, we might have
     * mismatched level sizes and the copies would fail.
@@ -98,7 +98,7 @@ intel_finalize_mipmap_tree(struct brw_context *brw, GLuint unit)
    if (!intelObj->needs_validate &&
        validate_first_level >= intelObj->validated_first_level &&
        validate_last_level <= intelObj->validated_last_level) {
-      return true;
+      return;
    }
 
    /* On recent generations, immutable textures should not get this far
@@ -136,8 +136,6 @@ intel_finalize_mipmap_tree(struct brw_context *brw, GLuint unit)
                  _mesa_get_format_name(firstImage->base.Base.TexFormat),
                  width, height, depth, validate_last_level + 1);
 
-      const uint32_t layout_flags = MIPTREE_LAYOUT_ACCELERATED_UPLOAD |
-                                    MIPTREE_LAYOUT_TILING_ANY;
       intelObj->mt = intel_miptree_create(brw,
                                           intelObj->base.Target,
 					  firstImage->base.Base.TexFormat,
@@ -146,10 +144,10 @@ intel_finalize_mipmap_tree(struct brw_context *brw, GLuint unit)
                                           width,
                                           height,
                                           depth,
-                                          0 /* num_samples */,
-                                          layout_flags);
+                                          1 /* num_samples */,
+                                          MIPTREE_CREATE_BUSY);
       if (!intelObj->mt)
-         return false;
+         return;
    }
 
    /* Pull in any images not in the object's tree:
@@ -180,6 +178,23 @@ intel_finalize_mipmap_tree(struct brw_context *brw, GLuint unit)
    intelObj->validated_last_level = validate_last_level;
    intelObj->_Format = intelObj->mt->format;
    intelObj->needs_validate = false;
+}
 
-   return true;
+/**
+ * Finalizes all textures, completing any rendering that needs to be done
+ * to prepare them.
+ */
+void
+brw_validate_textures(struct brw_context *brw)
+{
+   struct gl_context *ctx = &brw->ctx;
+   const int max_enabled_unit = ctx->Texture._MaxEnabledTexImageUnit;
+
+   for (int unit = 0; unit <= max_enabled_unit; unit++) {
+      struct gl_texture_unit *tex_unit = &ctx->Texture.Unit[unit];
+
+      if (tex_unit->_Current) {
+         intel_finalize_mipmap_tree(brw, unit);
+      }
+   }
 }
